"""
Created on Thu Nov 29 2022
Author: Anton Boev (Skoltech)
Note: Script allows to create a series of conformers for a molecule by rotation of a part of atoms
"""
import sys, re, copy
import numpy as np
from copy import deepcopy


input_filename = 'ethanole.lmpdat' # path to inp file with ideal structure with *.lmpdat extention obtained via Avogadro: Avogadro -> file -> export -> Molecule -> LAMMPS format -> OpenBabel format
atoms2rot = [8,9] # list of atoms that should be rotated (from 1), atoms should form a part of molecule
rot_axis = [5,8] # atom numbers forming a rotation axis, two atoms
angle_list = range(0,91,10) # rotation angle range (degree) (from, to, step)










def rot(vec, ax, angle):
    '''
    the function rotates vector around the axis
    input: 
    vec (list[3*float]) - vector that should be rotated
    ax (list[3*float]) - vector of the rotation axis 
    angle - rotation angle in degrees
    '''
    from scipy.spatial.transform import Rotation as R

    rotation_degrees = angle
    rotation_radians = np.radians(rotation_degrees)
    rotation_axis = np.array(ax)

    rotation_vector = rotation_radians * rotation_axis
    rotation = R.from_rotvec(rotation_vector)
    rotated_vec = rotation.apply(vec)

    return rotated_vec



def rotation(str_atoms, atoms2rot, angle, border_atom, ax = None,vec_btwn_atoms = None):
    '''
    the function rotates vector around the axis
    input: 
    st - siman structure object
    atoms2rot (list[n*float]) - list of atom numbers from zero that should be rotated 
    angle - rotation angle in degrees
    border_atom - number of atom around which rotation takes place
    ax (list[3*float]) - vector of the rotation axis 
    vec_btwn_atoms (list[2*float]) - list of atoms to create rotation vector if ax is not provided
    '''

    str_atoms_new = deepcopy(str_atoms)

    if ax is None:
        # print('Rotation axis is not provided, try to create the vector between two atoms')
        if vec_btwn_atoms:
            ax = np.array([float(i)-float(j) for i, j in zip(str_atoms[vec_btwn_atoms[0]-1].split()[4:7], str_atoms[vec_btwn_atoms[1]-1].split()[4:7])])
        else:
            print('Error! You have provided either rotation vector nor atoms to create one... Fix it please')


    for at in atoms2rot:
        vec = np.array([float(i)-float(j) for i, j in zip(str_atoms[at-1].split()[4:7], str_atoms[border_atom-1].split()[4:7])])
        rot_vec = rot(vec, ax, angle)
        str_atoms_new[at-1] = str_atoms[at-1].split()[:4]
        for i in range(0,3):
            n = float(rot_vec[i]) + float(str_atoms[border_atom-1].split()[4+i])
            n=round(n,5)
            str_atoms_new[at-1].append(str(n))

        str_atoms_new[at-1].extend(str_atoms[at-1].split()[7:])
        str_atoms_new[at-1] = ' '.join(str_atoms_new[at-1])+'\n'

    return str_atoms_new


#########################################
# PARAMETERS
#########################################
# atoms2rot = [1,2,3,9,10,11,12,13,14,25] # list of atoms that should be rotated (from 1)
# vec_btwn_atoms = [4,3] # atoms forming the bond of rotation axis



#############################
#Reading of the input file
#############################



str_header = []
str_atoms = []
str_footer = []
c_header = 1
c_atoms =0
c_footer = 0
with open(input_filename) as f:
    lines = [line for line in f]
    for line in lines:

        if c_header:
            str_header.append(line)
        if c_atoms:
            str_atoms.append(line)

        if 'Atoms' in line:
            c_header = 0
            c_atoms = 1
        
        if 'Bonds' in line:
            str_atoms = str_atoms[1:-2]
            c_header = 0
            c_atoms = 0
            c_footer = 1
        if c_footer:
            str_footer.append(line)





#########################################
#########################################


for angle in angle_list:
    str_atoms_new = rotation(str_atoms, atoms2rot, angle, border_atom = rot_axis[0], ax = None, vec_btwn_atoms = rot_axis)
    output_filename = input_filename.split('.')[0]+f'_{angle}.lmpdat'
    with open(output_filename, 'w') as fp:
        fp.write(' '.join(str_header)+'\n')
        fp.write(' '.join(str_atoms_new)+'\n')
        fp.write(' '.join(str_footer))


print(f'Rotation is performed successfully!\nThe last file is {output_filename}')

